{% if is_staff %}
<link rel="stylesheet" type="text/css" href="{{ INLINETRANS_MEDIA_URL }}css/inlinetrans.css"/>
<script language="javascript" src="{{ INLINETRANS_MEDIA_URL }}js/jquery.inlinetrans.js"></script>
{% endif %}
